#include "zu_objects.h"
#include "zu_resources.h"
#include "built_in.h"

// Extern memory interface
// MMC/SD Connections
sbit Mmc_Chip_Select at PORTF_OUT.B5;
sbit Mmc_Chip_Select_Direction at PORTF_DIR.B5;
// end of MMC/SD
// TFT Get Data globals
char Ext_Data_Buffer[512];
unsigned long currentSector = -1, res_file_size;
// end of TFT Get Data
// End of extern memory interface


// TFT module connections
char TFT_DataPort at PORTK_OUT;
sbit TFT_RST at PORTJ_OUT.B1;
sbit TFT_BLED at PORTC_OUT.B4;
sbit TFT_RS at PORTJ_OUT.B4;
sbit TFT_CS at PORTJ_OUT.B5;
sbit TFT_RD at PORTJ_OUT.B2;
sbit TFT_WR at PORTJ_OUT.B3;
char TFT_DataPort_Direction at PORTK_DIR;
sbit TFT_RST_Direction at PORTJ_DIR.B1;
sbit TFT_BLED_Direction at PORTC_DIR.B4;
sbit TFT_RS_Direction at PORTJ_DIR.B4;
sbit TFT_CS_Direction at PORTJ_DIR.B5;
sbit TFT_RD_Direction at PORTJ_DIR.B2;
sbit TFT_WR_Direction at PORTJ_DIR.B3;
// End TFT module connections

// Touch Panel module connections
sbit DRIVEX_LEFT at PORTA_OUT.B2;
sbit DRIVEX_RIGHT at PORTQ_OUT.B2;
sbit DRIVEY_UP at PORTQ_OUT.B3;
sbit DRIVEY_DOWN at PORTA_OUT.B1;
sbit DRIVEX_LEFT_DIRECTION at PORTA_DIR.B2;
sbit DRIVEX_RIGHT_DIRECTION at PORTQ_DIR.B2;
sbit DRIVEY_UP_DIRECTION at PORTQ_DIR.B3;
sbit DRIVEY_DOWN_DIRECTION at PORTA_DIR.B1;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 900;
char PenDown;
typedef unsigned long TPointer;
TPointer PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TButton *local_button;
TButton *exec_button;
int button_order;
TLabel *local_label;
TLabel *exec_label;
int label_order;
TImage *local_image;
TImage *exec_image;
int image_order;
TBox *local_box;
TBox *exec_box;
int box_order;
TBox_Round *local_round_box;
TBox_Round *exec_round_box;
int box_round_order;
TCBox_Round *local_round_cbox;
TCBox_Round *exec_round_cbox;
int cbox_round_order;


void Init_ADC() {
  PORTA_DIR = 0;
  PORTA_OUT = 0;
  ADCA_Init_Advanced(_ADC_Signed | _ADC_12bit, _ADC_EXTERNAL_REF_A);
  ADCA_PRESCALER = 2;              // system clock division factor div16
}

char* TFT_Get_Data(unsigned long offset, unsigned int count, unsigned int *num) {
unsigned long start_sector;
unsigned int pos;

  start_sector = Mmc_Get_File_Write_Sector() + offset/512;
  pos = (unsigned long)offset%512;

  if(start_sector == currentSector+1) {
    Mmc_Multi_Read_Sector(Ext_Data_Buffer);
    currentSector = start_sector;
  } else if (start_sector != currentSector) {
    if(currentSector != -1)
      Mmc_Multi_Read_Stop();
    Mmc_Multi_Read_Start(start_sector);
    Mmc_Multi_Read_Sector(Ext_Data_Buffer);
    currentSector = start_sector;
  }

  if(count>512-pos)
    *num = 512-pos;
  else
    *num = count;

  return Ext_Data_Buffer+pos;
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Init(320, 240);
  TFT_Set_Ext_Buffer(TFT_Get_Data);

  TP_TFT_Init(320, 240, 1, 2);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TImage               Image1;
  TLabel                 Label1;
char Label1_Caption[74] = "\xFA\xC1\xD2\xD1\xC4\xCE\xCF\xC5 \xF5\xD3\xD4\xD2\xCF\xCA\xD3\xD4\xD7\xCF";

  TImage               Image2;
  TLabel                 * const code far Screen1_Labels[1]=
         {
         &Label1               
         };
  TImage                 * const code far Screen1_Images[2]=
         {
         &Image1,              
         &Image2               
         };


  TScreen                Screen2;
  TImage               Image3;
  TImage               Image4;
  TImage               Image5;
  TImage                 * const code far Screen2_Images[3]=
         {
         &Image3,              
         &Image4,              
         &Image5               
         };


  TScreen                Screen3;
  TImage               Image6;
  TButton               Button13;
char Button13_Caption[2] = "1";

  TButton               Button1;
char Button1_Caption[2] = "2";

  TButton               Button2;
char Button2_Caption[2] = "3";

  TButton               Button3;
char Button3_Caption[2] = "4";

  TButton               Button4;
char Button4_Caption[2] = "5";

  TButton               Button5;
char Button5_Caption[2] = "6";

  TButton               Button6;
char Button6_Caption[2] = "7";

  TButton               Button7;
char Button7_Caption[2] = "8";

  TButton               Button8;
char Button8_Caption[2] = "9";

  TButton               Button9;
char Button9_Caption[4] = "Esc";

  TButton               Button10;
char Button10_Caption[6] = "Enter";

  TButton               Button11;
char Button11_Caption[2] = "0";

  TCBox_Round             BoxRound2 = 
         {
         &Screen3              , //   BoxRound2.OwnerScreen
         13                    , //   BoxRound2.Order           
         10                    , //   BoxRound2.Left           
         57                    , //   BoxRound2.Top             
         300                   , //   BoxRound2.Width           
         32                    , //   BoxRound2.Height          
         2                     , //   BoxRound2.Pen_Width       
         0xFC00                , //   BoxRound2.Pen_Color       
         1                     , //   BoxRound2.Visible         
         0                     , //   BoxRound2.Active          
         1                     , //   BoxRound2.Transparent     
         0                     , //   BoxRound2.Gradient        
         0                     , //   BoxRound2.Gradient_Orientation    
         0xFFFF                , //   BoxRound2.Gradient_Start_Color    
         0xC618                , //   BoxRound2.Gradient_End_Color      
         0xC618                , //   BoxRound2.Color           
         3                     , //   BoxRound2.CornerRadius           
         0                     , //   BoxRound2.PressColEnabled 
         0x8410                , //   BoxRound2.Press_Color     
         0                     ,//  BoxRound2.OnUpPtr
         0                     ,//  BoxRound2.OnDownPtr
         0                     ,//  BoxRound2.OnClickPtr
         0                      //  BoxRound2.OnPressPtr
         };
  TButton                * const code far Screen3_Buttons[12]=
         {
         &Button13,            
         &Button1,             
         &Button2,             
         &Button3,             
         &Button4,             
         &Button5,             
         &Button6,             
         &Button7,             
         &Button8,             
         &Button9,             
         &Button10,            
         &Button11             
         };
  TImage                 * const code far Screen3_Images[1]=
         {
         &Image6               
         };
  TCBox_Round             * const code far Screen3_CBoxes_Round[1]=
         {
         &BoxRound2            
         };


  TScreen                Screen4;
  TImage               Image7;
  TButton               Button12;
char Button12_Caption[2] = "-";

  TButton               Button14;
char Button14_Caption[2] = "+";

  TBox                   Box5;
  TBox                   Box1;
  TBox                   Box2;
  TBox                   Box3;
  TBox_Round             BoxRound1;
  TButton                * const code far Screen4_Buttons[2]=
         {
         &Button12,            
         &Button14             
         };
  TImage                 * const code far Screen4_Images[1]=
         {
         &Image7               
         };
  TBox                   * const code far Screen4_Boxes[4]=
         {
         &Box5,                
         &Box1,                
         &Box2,                
         &Box3                 
         };
  TBox_Round             * const code far Screen4_Boxes_Round[1]=
         {
         &BoxRound1            
         };




static void InitializeObjects() {
  Screen1.Color                     = 0x0000;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.ButtonsCount              = 0;
  Screen1.LabelsCount               = 1;
  Screen1.Labels                    = Screen1_Labels;
  Screen1.ImagesCount               = 2;
  Screen1.Images                    = Screen1_Images;
  Screen1.BoxesCount                = 0;
  Screen1.Boxes_RoundCount          = 0;
  Screen1.CBoxes_RoundCount          = 0;
  Screen1.ObjectsCount              = 3;

  Screen2.Color                     = 0x0000;
  Screen2.Width                     = 320;
  Screen2.Height                    = 240;
  Screen2.ButtonsCount              = 0;
  Screen2.LabelsCount               = 0;
  Screen2.ImagesCount               = 3;
  Screen2.Images                    = Screen2_Images;
  Screen2.BoxesCount                = 0;
  Screen2.Boxes_RoundCount          = 0;
  Screen2.CBoxes_RoundCount          = 0;
  Screen2.ObjectsCount              = 3;

  Screen3.Color                     = 0x0000;
  Screen3.Width                     = 320;
  Screen3.Height                    = 240;
  Screen3.ButtonsCount              = 12;
  Screen3.Buttons                   = Screen3_Buttons;
  Screen3.LabelsCount               = 0;
  Screen3.ImagesCount               = 1;
  Screen3.Images                    = Screen3_Images;
  Screen3.BoxesCount                = 0;
  Screen3.Boxes_RoundCount          = 0;
  Screen3.CBoxes_RoundCount          = 1;
  Screen3.CBoxes_Round               = Screen3_CBoxes_Round;
  Screen3.ObjectsCount              = 14;

  Screen4.Color                     = 0x0000;
  Screen4.Width                     = 320;
  Screen4.Height                    = 240;
  Screen4.ButtonsCount              = 2;
  Screen4.Buttons                   = Screen4_Buttons;
  Screen4.LabelsCount               = 0;
  Screen4.ImagesCount               = 1;
  Screen4.Images                    = Screen4_Images;
  Screen4.BoxesCount                = 4;
  Screen4.Boxes                     = Screen4_Boxes;
  Screen4.Boxes_RoundCount          = 1;
  Screen4.Boxes_Round               = Screen4_Boxes_Round;
  Screen4.CBoxes_RoundCount          = 0;
  Screen4.ObjectsCount              = 8;


  Image1.OwnerScreen     = &Screen1;
  Image1.Order          = 0;
  Image1.Left           = 1;
  Image1.Top            = 1;
  Image1.Width          = 320;
  Image1.Height         = 239;
  Image1.Picture_Type   = 0;
  Image1.Picture_Ratio  = 1;
  Image1.Picture_Name   = slon_bmp;
  Image1.Visible        = 1;
  Image1.Active         = 1;
  Image1.OnUpPtr         = 0;
  Image1.OnDownPtr       = 0;
  Image1.OnClickPtr      = 0;
  Image1.OnPressPtr      = 0;

  Label1.OwnerScreen     = &Screen1;
  Label1.Order          = 1;
  Label1.Left           = 8;
  Label1.Top            = 8;
  Label1.Width          = 541;
  Label1.Height         = 15;
  Label1.Visible        = 1;
  Label1.Active         = 1;
  Label1.Caption        = Label1_Caption;
  Label1.FontName       = Lucida_Console_KOI89x14_Bold;
  Label1.Font_Color     = 0xFFFF;
  Label1.OnUpPtr         = 0;
  Label1.OnDownPtr       = 0;
  Label1.OnClickPtr      = 0;
  Label1.OnPressPtr      = 0;

  Image2.OwnerScreen     = &Screen1;
  Image2.Order          = 2;
  Image2.Left           = 236;
  Image2.Top            = 164;
  Image2.Width          = 60;
  Image2.Height         = 56;
  Image2.Picture_Type   = 0;
  Image2.Picture_Ratio  = 1;
  Image2.Picture_Name   = settings_bmp;
  Image2.Visible        = 1;
  Image2.Active         = 1;
  Image2.OnUpPtr         = 0;
  Image2.OnDownPtr       = 0;
  Image2.OnClickPtr      = 0;
  Image2.OnPressPtr      = setting;

  Image3.OwnerScreen     = &Screen2;
  Image3.Order          = 0;
  Image3.Left           = 18;
  Image3.Top            = 18;
  Image3.Width          = 40;
  Image3.Height         = 39;
  Image3.Picture_Type   = 0;
  Image3.Picture_Ratio  = 1;
  Image3.Picture_Name   = clock_bmp;
  Image3.Visible        = 1;
  Image3.Active         = 1;
  Image3.OnUpPtr         = 0;
  Image3.OnDownPtr       = 0;
  Image3.OnClickPtr      = 0;
  Image3.OnPressPtr      = clock;

  Image4.OwnerScreen     = &Screen2;
  Image4.Order          = 1;
  Image4.Left           = 83;
  Image4.Top            = 18;
  Image4.Width          = 52;
  Image4.Height         = 40;
  Image4.Picture_Type   = 0;
  Image4.Picture_Ratio  = 1;
  Image4.Picture_Name   = lampa_bmp;
  Image4.Visible        = 1;
  Image4.Active         = 1;
  Image4.OnUpPtr         = 0;
  Image4.OnDownPtr       = 0;
  Image4.OnClickPtr      = 0;
  Image4.OnPressPtr      = lamp;

  Image5.OwnerScreen     = &Screen2;
  Image5.Order          = 2;
  Image5.Left           = 264;
  Image5.Top            = 181;
  Image5.Width          = 37;
  Image5.Height         = 40;
  Image5.Picture_Type   = 0;
  Image5.Picture_Ratio  = 1;
  Image5.Picture_Name   = out_bmp;
  Image5.Visible        = 1;
  Image5.Active         = 1;
  Image5.OnUpPtr         = 0;
  Image5.OnDownPtr       = 0;
  Image5.OnClickPtr      = 0;
  Image5.OnPressPtr      = exit;

  Image6.OwnerScreen     = &Screen3;
  Image6.Order          = 0;
  Image6.Left           = 270;
  Image6.Top            = 10;
  Image6.Width          = 37;
  Image6.Height         = 40;
  Image6.Picture_Type   = 0;
  Image6.Picture_Ratio  = 1;
  Image6.Picture_Name   = out_bmp;
  Image6.Visible        = 1;
  Image6.Active         = 1;
  Image6.OnUpPtr         = 0;
  Image6.OnDownPtr       = 0;
  Image6.OnClickPtr      = 0;
  Image6.OnPressPtr      = exit2;

  Button13.OwnerScreen     = &Screen3;
  Button13.Order           = 1;
  Button13.Left            = 10;
  Button13.Top             = 104;
  Button13.Width           = 60;
  Button13.Height          = 30;
  Button13.Pen_Width       = 2;
  Button13.Pen_Color       = 0x0400;
  Button13.Visible         = 1;
  Button13.Active          = 1;
  Button13.Transparent     = 1;
  Button13.Caption         = Button13_Caption;
  Button13.TextAlign             = _taCenter;
  Button13.FontName        = Lucida_Console_KOI812x20_Bold;
  Button13.PressColEnabled = 1;
  Button13.Font_Color      = 0x0000;
  Button13.Gradient        = 1;
  Button13.Gradient_Orientation    = 0;
  Button13.Gradient_Start_Color    = 0xFFFF;
  Button13.Gradient_End_Color      = 0xFC00;
  Button13.Color           = 0xFC00;
  Button13.Press_Color     = 0xFC00;
  Button13.OnUpPtr         = 0;
  Button13.OnDownPtr       = 0;
  Button13.OnClickPtr      = odin;
  Button13.OnPressPtr      = 0;

  Button1.OwnerScreen     = &Screen3;
  Button1.Order           = 2;
  Button1.Left            = 90;
  Button1.Top             = 104;
  Button1.Width           = 60;
  Button1.Height          = 30;
  Button1.Pen_Width       = 2;
  Button1.Pen_Color       = 0x0400;
  Button1.Visible         = 1;
  Button1.Active          = 1;
  Button1.Transparent     = 1;
  Button1.Caption         = Button1_Caption;
  Button1.TextAlign             = _taCenter;
  Button1.FontName        = Lucida_Console_KOI812x20_Bold;
  Button1.PressColEnabled = 1;
  Button1.Font_Color      = 0x0000;
  Button1.Gradient        = 1;
  Button1.Gradient_Orientation    = 0;
  Button1.Gradient_Start_Color    = 0xFFFF;
  Button1.Gradient_End_Color      = 0xFC00;
  Button1.Color           = 0xFC00;
  Button1.Press_Color     = 0xFC00;
  Button1.OnUpPtr         = 0;
  Button1.OnDownPtr       = 0;
  Button1.OnClickPtr      = dva;
  Button1.OnPressPtr      = 0;

  Button2.OwnerScreen     = &Screen3;
  Button2.Order           = 3;
  Button2.Left            = 170;
  Button2.Top             = 104;
  Button2.Width           = 60;
  Button2.Height          = 30;
  Button2.Pen_Width       = 2;
  Button2.Pen_Color       = 0x0400;
  Button2.Visible         = 1;
  Button2.Active          = 1;
  Button2.Transparent     = 1;
  Button2.Caption         = Button2_Caption;
  Button2.TextAlign             = _taCenter;
  Button2.FontName        = Lucida_Console_KOI812x20_Bold;
  Button2.PressColEnabled = 1;
  Button2.Font_Color      = 0x0000;
  Button2.Gradient        = 1;
  Button2.Gradient_Orientation    = 0;
  Button2.Gradient_Start_Color    = 0xFFFF;
  Button2.Gradient_End_Color      = 0xFC00;
  Button2.Color           = 0xFC00;
  Button2.Press_Color     = 0xFC00;
  Button2.OnUpPtr         = 0;
  Button2.OnDownPtr       = 0;
  Button2.OnClickPtr      = tri;
  Button2.OnPressPtr      = 0;

  Button3.OwnerScreen     = &Screen3;
  Button3.Order           = 4;
  Button3.Left            = 250;
  Button3.Top             = 104;
  Button3.Width           = 60;
  Button3.Height          = 30;
  Button3.Pen_Width       = 2;
  Button3.Pen_Color       = 0x0400;
  Button3.Visible         = 1;
  Button3.Active          = 1;
  Button3.Transparent     = 1;
  Button3.Caption         = Button3_Caption;
  Button3.TextAlign             = _taCenter;
  Button3.FontName        = Lucida_Console_KOI812x20_Bold;
  Button3.PressColEnabled = 1;
  Button3.Font_Color      = 0x0000;
  Button3.Gradient        = 1;
  Button3.Gradient_Orientation    = 0;
  Button3.Gradient_Start_Color    = 0xFFFF;
  Button3.Gradient_End_Color      = 0xFC00;
  Button3.Color           = 0xFC00;
  Button3.Press_Color     = 0xFC00;
  Button3.OnUpPtr         = 0;
  Button3.OnDownPtr       = 0;
  Button3.OnClickPtr      = chetire;
  Button3.OnPressPtr      = 0;

  Button4.OwnerScreen     = &Screen3;
  Button4.Order           = 5;
  Button4.Left            = 10;
  Button4.Top             = 154;
  Button4.Width           = 60;
  Button4.Height          = 30;
  Button4.Pen_Width       = 2;
  Button4.Pen_Color       = 0x0400;
  Button4.Visible         = 1;
  Button4.Active          = 1;
  Button4.Transparent     = 1;
  Button4.Caption         = Button4_Caption;
  Button4.TextAlign             = _taCenter;
  Button4.FontName        = Lucida_Console_KOI812x20_Bold;
  Button4.PressColEnabled = 1;
  Button4.Font_Color      = 0x0000;
  Button4.Gradient        = 1;
  Button4.Gradient_Orientation    = 0;
  Button4.Gradient_Start_Color    = 0xFFFF;
  Button4.Gradient_End_Color      = 0xFC00;
  Button4.Color           = 0xFC00;
  Button4.Press_Color     = 0xFC00;
  Button4.OnUpPtr         = 0;
  Button4.OnDownPtr       = 0;
  Button4.OnClickPtr      = pyat;
  Button4.OnPressPtr      = 0;

  Button5.OwnerScreen     = &Screen3;
  Button5.Order           = 6;
  Button5.Left            = 90;
  Button5.Top             = 154;
  Button5.Width           = 60;
  Button5.Height          = 30;
  Button5.Pen_Width       = 2;
  Button5.Pen_Color       = 0x0400;
  Button5.Visible         = 1;
  Button5.Active          = 1;
  Button5.Transparent     = 1;
  Button5.Caption         = Button5_Caption;
  Button5.TextAlign             = _taCenter;
  Button5.FontName        = Lucida_Console_KOI812x20_Bold;
  Button5.PressColEnabled = 1;
  Button5.Font_Color      = 0x0000;
  Button5.Gradient        = 1;
  Button5.Gradient_Orientation    = 0;
  Button5.Gradient_Start_Color    = 0xFFFF;
  Button5.Gradient_End_Color      = 0xFC00;
  Button5.Color           = 0xFC00;
  Button5.Press_Color     = 0xFC00;
  Button5.OnUpPtr         = 0;
  Button5.OnDownPtr       = 0;
  Button5.OnClickPtr      = chest;
  Button5.OnPressPtr      = 0;

  Button6.OwnerScreen     = &Screen3;
  Button6.Order           = 7;
  Button6.Left            = 170;
  Button6.Top             = 154;
  Button6.Width           = 60;
  Button6.Height          = 30;
  Button6.Pen_Width       = 2;
  Button6.Pen_Color       = 0x0400;
  Button6.Visible         = 1;
  Button6.Active          = 1;
  Button6.Transparent     = 1;
  Button6.Caption         = Button6_Caption;
  Button6.TextAlign             = _taCenter;
  Button6.FontName        = Lucida_Console_KOI812x20_Bold;
  Button6.PressColEnabled = 1;
  Button6.Font_Color      = 0x0000;
  Button6.Gradient        = 1;
  Button6.Gradient_Orientation    = 0;
  Button6.Gradient_Start_Color    = 0xFFFF;
  Button6.Gradient_End_Color      = 0xFC00;
  Button6.Color           = 0xFC00;
  Button6.Press_Color     = 0xFC00;
  Button6.OnUpPtr         = 0;
  Button6.OnDownPtr       = 0;
  Button6.OnClickPtr      = sem;
  Button6.OnPressPtr      = 0;

  Button7.OwnerScreen     = &Screen3;
  Button7.Order           = 8;
  Button7.Left            = 250;
  Button7.Top             = 154;
  Button7.Width           = 60;
  Button7.Height          = 30;
  Button7.Pen_Width       = 2;
  Button7.Pen_Color       = 0x0400;
  Button7.Visible         = 1;
  Button7.Active          = 1;
  Button7.Transparent     = 1;
  Button7.Caption         = Button7_Caption;
  Button7.TextAlign             = _taCenter;
  Button7.FontName        = Lucida_Console_KOI812x20_Bold;
  Button7.PressColEnabled = 1;
  Button7.Font_Color      = 0x0000;
  Button7.Gradient        = 1;
  Button7.Gradient_Orientation    = 0;
  Button7.Gradient_Start_Color    = 0xFFFF;
  Button7.Gradient_End_Color      = 0xFC00;
  Button7.Color           = 0xFC00;
  Button7.Press_Color     = 0xFC00;
  Button7.OnUpPtr         = 0;
  Button7.OnDownPtr       = 0;
  Button7.OnClickPtr      = vosem;
  Button7.OnPressPtr      = 0;

  Button8.OwnerScreen     = &Screen3;
  Button8.Order           = 9;
  Button8.Left            = 10;
  Button8.Top             = 203;
  Button8.Width           = 60;
  Button8.Height          = 30;
  Button8.Pen_Width       = 2;
  Button8.Pen_Color       = 0x0400;
  Button8.Visible         = 1;
  Button8.Active          = 1;
  Button8.Transparent     = 1;
  Button8.Caption         = Button8_Caption;
  Button8.TextAlign             = _taCenter;
  Button8.FontName        = Lucida_Console_KOI812x20_Bold;
  Button8.PressColEnabled = 1;
  Button8.Font_Color      = 0x0000;
  Button8.Gradient        = 1;
  Button8.Gradient_Orientation    = 0;
  Button8.Gradient_Start_Color    = 0xFFFF;
  Button8.Gradient_End_Color      = 0xFC00;
  Button8.Color           = 0xFC00;
  Button8.Press_Color     = 0xFC00;
  Button8.OnUpPtr         = 0;
  Button8.OnDownPtr       = 0;
  Button8.OnClickPtr      = devyat;
  Button8.OnPressPtr      = 0;

  Button9.OwnerScreen     = &Screen3;
  Button9.Order           = 10;
  Button9.Left            = 90;
  Button9.Top             = 203;
  Button9.Width           = 60;
  Button9.Height          = 30;
  Button9.Pen_Width       = 2;
  Button9.Pen_Color       = 0x0400;
  Button9.Visible         = 1;
  Button9.Active          = 1;
  Button9.Transparent     = 1;
  Button9.Caption         = Button9_Caption;
  Button9.TextAlign             = _taCenter;
  Button9.FontName        = Lucida_Console_KOI812x20_Bold;
  Button9.PressColEnabled = 1;
  Button9.Font_Color      = 0x0000;
  Button9.Gradient        = 1;
  Button9.Gradient_Orientation    = 0;
  Button9.Gradient_Start_Color    = 0xFFFF;
  Button9.Gradient_End_Color      = 0xFC00;
  Button9.Color           = 0xFC00;
  Button9.Press_Color     = 0xFC00;
  Button9.OnUpPtr         = 0;
  Button9.OnDownPtr       = 0;
  Button9.OnClickPtr      = esc;
  Button9.OnPressPtr      = 0;

  Button10.OwnerScreen     = &Screen3;
  Button10.Order           = 11;
  Button10.Left            = 170;
  Button10.Top             = 203;
  Button10.Width           = 60;
  Button10.Height          = 30;
  Button10.Pen_Width       = 2;
  Button10.Pen_Color       = 0x0400;
  Button10.Visible         = 1;
  Button10.Active          = 1;
  Button10.Transparent     = 1;
  Button10.Caption         = Button10_Caption;
  Button10.TextAlign             = _taCenter;
  Button10.FontName        = Lucida_Console_KOI89x14_Bold;
  Button10.PressColEnabled = 1;
  Button10.Font_Color      = 0x0000;
  Button10.Gradient        = 1;
  Button10.Gradient_Orientation    = 0;
  Button10.Gradient_Start_Color    = 0xFFFF;
  Button10.Gradient_End_Color      = 0xFC00;
  Button10.Color           = 0xFC00;
  Button10.Press_Color     = 0xFC00;
  Button10.OnUpPtr         = 0;
  Button10.OnDownPtr       = 0;
  Button10.OnClickPtr      = enter;
  Button10.OnPressPtr      = 0;

  Button11.OwnerScreen     = &Screen3;
  Button11.Order           = 12;
  Button11.Left            = 250;
  Button11.Top             = 203;
  Button11.Width           = 60;
  Button11.Height          = 30;
  Button11.Pen_Width       = 2;
  Button11.Pen_Color       = 0x0400;
  Button11.Visible         = 1;
  Button11.Active          = 1;
  Button11.Transparent     = 1;
  Button11.Caption         = Button11_Caption;
  Button11.TextAlign             = _taCenter;
  Button11.FontName        = Lucida_Console_KOI812x20_Bold;
  Button11.PressColEnabled = 1;
  Button11.Font_Color      = 0x0000;
  Button11.Gradient        = 1;
  Button11.Gradient_Orientation    = 0;
  Button11.Gradient_Start_Color    = 0xFFFF;
  Button11.Gradient_End_Color      = 0xFC00;
  Button11.Color           = 0xFC00;
  Button11.Press_Color     = 0xFC00;
  Button11.OnUpPtr         = 0;
  Button11.OnDownPtr       = 0;
  Button11.OnClickPtr      = nol;
  Button11.OnPressPtr      = 0;

  Image7.OwnerScreen     = &Screen4;
  Image7.Order          = 0;
  Image7.Left           = 270;
  Image7.Top            = 10;
  Image7.Width          = 37;
  Image7.Height         = 40;
  Image7.Picture_Type   = 0;
  Image7.Picture_Ratio  = 1;
  Image7.Picture_Name   = out_bmp;
  Image7.Visible        = 1;
  Image7.Active         = 1;
  Image7.OnUpPtr         = 0;
  Image7.OnDownPtr       = 0;
  Image7.OnClickPtr      = 0;
  Image7.OnPressPtr      = exit3;

  Button12.OwnerScreen     = &Screen4;
  Button12.Order           = 1;
  Button12.Left            = 20;
  Button12.Top             = 100;
  Button12.Width           = 50;
  Button12.Height          = 25;
  Button12.Pen_Width       = 1;
  Button12.Pen_Color       = 0x0000;
  Button12.Visible         = 1;
  Button12.Active          = 1;
  Button12.Transparent     = 1;
  Button12.Caption         = Button12_Caption;
  Button12.TextAlign             = _taCenter;
  Button12.FontName        = Lucida_Console_KOI89x14_Bold;
  Button12.PressColEnabled = 1;
  Button12.Font_Color      = 0x0000;
  Button12.Gradient        = 1;
  Button12.Gradient_Orientation    = 0;
  Button12.Gradient_Start_Color    = 0xFFFF;
  Button12.Gradient_End_Color      = 0xC618;
  Button12.Color           = 0xC618;
  Button12.Press_Color     = 0xC618;
  Button12.OnUpPtr         = 0;
  Button12.OnDownPtr       = 0;
  Button12.OnClickPtr      = 0;
  Button12.OnPressPtr      = minus;

  Button14.OwnerScreen     = &Screen4;
  Button14.Order           = 2;
  Button14.Left            = 252;
  Button14.Top             = 100;
  Button14.Width           = 50;
  Button14.Height          = 25;
  Button14.Pen_Width       = 1;
  Button14.Pen_Color       = 0x0000;
  Button14.Visible         = 1;
  Button14.Active          = 1;
  Button14.Transparent     = 1;
  Button14.Caption         = Button14_Caption;
  Button14.TextAlign             = _taCenter;
  Button14.FontName        = Lucida_Console_KOI89x14_Bold;
  Button14.PressColEnabled = 1;
  Button14.Font_Color      = 0x0000;
  Button14.Gradient        = 1;
  Button14.Gradient_Orientation    = 0;
  Button14.Gradient_Start_Color    = 0xFFFF;
  Button14.Gradient_End_Color      = 0xC618;
  Button14.Color           = 0xC618;
  Button14.Press_Color     = 0xC618;
  Button14.OnUpPtr         = 0;
  Button14.OnDownPtr       = 0;
  Button14.OnClickPtr      = 0;
  Button14.OnPressPtr      = plus;

  Box5.OwnerScreen     = &Screen4;
  Box5.Order           = 3;
  Box5.Left            = 16;
  Box5.Top             = 160;
  Box5.Width           = 84;
  Box5.Height          = 54;
  Box5.Pen_Width       = 1;
  Box5.Pen_Color       = 0x0000;
  Box5.Visible         = 1;
  Box5.Active          = 1;
  Box5.Transparent     = 1;
  Box5.Gradient        = 0;
  Box5.Gradient_Orientation    = 0;
  Box5.Gradient_Start_Color    = 0xFFFF;
  Box5.Gradient_End_Color      = 0xC618;
  Box5.Color           = 0xF800;
  Box5.PressColEnabled     = 1;
  Box5.Press_Color     = 0xF81F;
  Box5.OnUpPtr         = 0;
  Box5.OnDownPtr       = 0;
  Box5.OnClickPtr      = 0;
  Box5.OnPressPtr      = 0;

  Box1.OwnerScreen     = &Screen4;
  Box1.Order           = 4;
  Box1.Left            = 120;
  Box1.Top             = 160;
  Box1.Width           = 84;
  Box1.Height          = 54;
  Box1.Pen_Width       = 1;
  Box1.Pen_Color       = 0x0000;
  Box1.Visible         = 1;
  Box1.Active          = 1;
  Box1.Transparent     = 1;
  Box1.Gradient        = 0;
  Box1.Gradient_Orientation    = 0;
  Box1.Gradient_Start_Color    = 0xFFFF;
  Box1.Gradient_End_Color      = 0xC618;
  Box1.Color           = 0x07E0;
  Box1.PressColEnabled     = 1;
  Box1.Press_Color     = 0xFFE0;
  Box1.OnUpPtr         = 0;
  Box1.OnDownPtr       = 0;
  Box1.OnClickPtr      = 0;
  Box1.OnPressPtr      = 0;

  Box2.OwnerScreen     = &Screen4;
  Box2.Order           = 5;
  Box2.Left            = 220;
  Box2.Top             = 160;
  Box2.Width           = 84;
  Box2.Height          = 54;
  Box2.Pen_Width       = 1;
  Box2.Pen_Color       = 0x0000;
  Box2.Visible         = 1;
  Box2.Active          = 1;
  Box2.Transparent     = 1;
  Box2.Gradient        = 0;
  Box2.Gradient_Orientation    = 0;
  Box2.Gradient_Start_Color    = 0xFFFF;
  Box2.Gradient_End_Color      = 0xC618;
  Box2.Color           = 0x001F;
  Box2.PressColEnabled     = 1;
  Box2.Press_Color     = 0x07FF;
  Box2.OnUpPtr         = 0;
  Box2.OnDownPtr       = 0;
  Box2.OnClickPtr      = 0;
  Box2.OnPressPtr      = 0;

  Box3.OwnerScreen     = &Screen4;
  Box3.Order           = 6;
  Box3.Left            = 120;
  Box3.Top             = 84;
  Box3.Width           = 84;
  Box3.Height          = 54;
  Box3.Pen_Width       = 1;
  Box3.Pen_Color       = 0x0000;
  Box3.Visible         = 1;
  Box3.Active          = 1;
  Box3.Transparent     = 1;
  Box3.Gradient        = 0;
  Box3.Gradient_Orientation    = 0;
  Box3.Gradient_Start_Color    = 0xFFFF;
  Box3.Gradient_End_Color      = 0xC618;
  Box3.Color           = 0xFFFF;
  Box3.PressColEnabled     = 1;
  Box3.Press_Color     = 0x8410;
  Box3.OnUpPtr         = 0;
  Box3.OnDownPtr       = 0;
  Box3.OnClickPtr      = 0;
  Box3.OnPressPtr      = 0;

  BoxRound1.OwnerScreen     = &Screen4;
  BoxRound1.Order           = 7;
  BoxRound1.Left            = 80;
  BoxRound1.Top             = 25;
  BoxRound1.Width           = 158;
  BoxRound1.Height          = 20;
  BoxRound1.Pen_Width       = 2;
  BoxRound1.Pen_Color       = 0x0400;
  BoxRound1.Visible         = 1;
  BoxRound1.Active          = 1;
  BoxRound1.Transparent     = 1;
  BoxRound1.Gradient        = 1;
  BoxRound1.Gradient_Orientation    = 0;
  BoxRound1.Gradient_Start_Color    = 0xFDD2;
  BoxRound1.Gradient_End_Color      = 0xFC00;
  BoxRound1.Color           = 0xFC00;
  BoxRound1.PressColEnabled     = 0;
  BoxRound1.Press_Color     = 0xFC08;
  BoxRound1.Corner_Radius      = 3;
  BoxRound1.OnUpPtr         = 0;
  BoxRound1.OnDownPtr       = 0;
  BoxRound1.OnClickPtr      = 0;
  BoxRound1.OnPressPtr      = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetButton(index)              CurrentScreen->Buttons[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetImage(index)               CurrentScreen->Images[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]
#define GetBox_Round(index)           CurrentScreen->Boxes_Round[index]
#define GetCBox_Round(index)          CurrentScreen->CBoxes_Round[index]

 void DeleteTrailingSpaces(char* str){
   char i;
   i = 0;
   while(1) {
   if(str[0] == ' ') {
      for(i = 0; i < strlen(str); i++) {
       str[i] = str[i+1];
      }
   }
   else
     break;
  }
 }

void DrawButton(TButton *Abutton) {
  if (Abutton->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(Abutton->Transparent, Abutton->Press_Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_End_Color, Abutton->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    }
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    TFT_Set_Ext_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    if (AButton->TextAlign == _taLeft)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + 4, (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (AButton->TextAlign == _taCenter)
      TFT_Write_Text(Abutton->Caption, (Abutton->Left + (Abutton->Width - caption_length) / 2), (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (AButton->TextAlign == _taRight)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + (Abutton->Width - caption_length - 4), (Abutton->Top + (Abutton->Height - caption_height) / 2));
  }
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Ext_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawImage(TImage *AImage) {
  if (AImage->Visible) {
    TFT_Ext_Image(AImage->Left, AImage->Top, AImage->Picture_Name, AImage->Picture_Ratio);
  }
}

void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ABox->Transparent, ABox->Press_Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_End_Color, ABox->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    }
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

void DrawRoundBox(TBox_Round *Around_box) {
    if (Around_box->Visible == 1) {
      if (object_pressed == 1) {
        object_pressed = 0;
        TFT_Set_Brush(Around_box->Transparent, Around_box->Press_Color, Around_box->Gradient, Around_box->Gradient_Orientation,
                      Around_box->Gradient_End_Color, Around_box->Gradient_Start_Color);
      }
      else {
        TFT_Set_Brush(Around_box->Transparent, Around_box->Color, Around_box->Gradient, Around_box->Gradient_Orientation,
                      Around_box->Gradient_Start_Color, Around_box->Gradient_End_Color);
      }
      TFT_Set_Pen(Around_box->Pen_Color, Around_box->Pen_Width);
      if (Around_box->Height > Around_box->Width) {
        TFT_Rectangle_Round_Edges(Around_box->Left + 1, Around_box->Top + 1,
          Around_box->Left + Around_box->Width - 2,
          Around_box->Top + Around_box->Height - 2, Around_box->Corner_Radius);
      }
      else
        {
          TFT_Rectangle_Round_Edges(Around_box->Left + 1, Around_box->Top + 1,
            Around_box->Left + Around_box->Width - 2,
            Around_box->Top + Around_box->Height - 2, Around_box->Corner_Radius);
        }
    }
}

void DrawCRoundBox(TCBox_Round *Around_box) {
    if (Around_box->Visible == 1) {
      if (object_pressed == 1) {
        object_pressed = 0;
        TFT_Set_Brush(Around_box->Transparent, Around_box->Press_Color, Around_box->Gradient, Around_box->Gradient_Orientation,
                      Around_box->Gradient_End_Color, Around_box->Gradient_Start_Color);
      }
      else {
        TFT_Set_Brush(Around_box->Transparent, Around_box->Color, Around_box->Gradient, Around_box->Gradient_Orientation,
                      Around_box->Gradient_Start_Color, Around_box->Gradient_End_Color);
      }
      TFT_Set_Pen(Around_box->Pen_Color, Around_box->Pen_Width);
      if (Around_box->Height > Around_box->Width) {
        TFT_Rectangle_Round_Edges(Around_box->Left + 1, Around_box->Top + 1,
          Around_box->Left + Around_box->Width - 2,
          Around_box->Top + Around_box->Height - 2, Around_box->Corner_Radius);
      }
      else
        {
          TFT_Rectangle_Round_Edges(Around_box->Left + 1, Around_box->Top + 1,
            Around_box->Left + Around_box->Width - 2,
            Around_box->Top + Around_box->Height - 2, Around_box->Corner_Radius);
        }
    }
}

void DrawScreen(TScreen *aScreen) {
 int order;
  unsigned short button_idx;
  TButton *local_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short box_idx;
  TBox *local_box;
  unsigned short round_box_idx;
  TBox_Round *local_round_box;
  unsigned short round_cbox_idx;
  TCBox_Round *local_round_cbox;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  button_idx = 0;
  label_idx = 0;
  image_idx = 0;
  box_idx = 0;
  round_box_idx = 0;
  round_cbox_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    save_bled_direction = TFT_BLED_Direction;
    TFT_BLED_Direction = 1;
    TFT_BLED           = 0;
    TFT_Init(CurrentScreen->Width, CurrentScreen->Height);
    TFT_Set_Ext_Buffer(TFT_Get_Data);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 1, 2);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
    TFT_BLED_Direction = save_bled_direction;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (button_idx < CurrentScreen->ButtonsCount) {
      local_button = GetButton(button_idx);
      if (order == local_button->Order) {
        button_idx++;
        order++;
        DrawButton(local_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

    if (round_box_idx < CurrentScreen->Boxes_RoundCount) {
      local_round_box = GetBox_Round(round_box_idx);
      if (order == local_round_box->Order) {
        round_box_idx++;
        order++;
        DrawRoundBox(local_round_box);
      }
    }

    if (round_cbox_idx < CurrentScreen->CBoxes_RoundCount) {
      local_round_cbox = GetCBox_Round(round_cbox_idx);
      if (order == local_round_cbox->Order) {
        round_cbox_idx++;
        order++;
        DrawCRoundBox(local_round_cbox);
      }
    }

    if (image_idx  < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  button_order        = -1;
  label_order         = -1;
  image_order         = -1;
  box_order           = -1;
  box_round_order     = -1;
  cbox_round_order    = -1;
  //  Buttons
  for ( _object_count = 0 ; _object_count < CurrentScreen->ButtonsCount ; _object_count++ ) {
    local_button = GetButton(_object_count);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  //  Labels
  for ( _object_count = 0 ; _object_count < CurrentScreen->LabelsCount ; _object_count++ ) {
    local_label = GetLabel(_object_count);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  //  Images
  for ( _object_count = 0 ; _object_count < CurrentScreen->ImagesCount ; _object_count++ ) {
    local_image = GetImage(_object_count);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  //  Boxes
  for ( _object_count = 0 ; _object_count < CurrentScreen->BoxesCount ; _object_count++ ) {
    local_box = GetBox(_object_count);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  //  Boxes with Round Edges
  for ( _object_count = 0 ; _object_count < CurrentScreen->Boxes_RoundCount ; _object_count++ ) {
    local_round_box = GetBox_Round(_object_count);
    if (local_round_box->Active == 1) {
      if (IsInsideObject(X, Y, local_round_box->Left, local_round_box->Top,
                         local_round_box->Width, local_round_box->Height) == 1) {
        box_round_order = local_round_box->Order;
        exec_round_box = local_round_box;
      }
    }
  }

  //  CBoxes with Round Edges
  for ( _object_count = 0 ; _object_count < CurrentScreen->CBoxes_RoundCount ; _object_count++ ) {
    local_round_cbox = GetCBox_Round(_object_count);
    if (local_round_cbox->Active == 1) {
      if (IsInsideObject(X, Y, local_round_cbox->Left, local_round_cbox->Top,
                         local_round_cbox->Width, local_round_cbox->Height) == 1) {
        cbox_round_order = local_round_cbox->Order;
        exec_round_cbox = local_round_cbox;
      }
    }
  }

  _object_count = -1;
  if (button_order > _object_count)
    _object_count = button_order;
  if (label_order >  _object_count )
    _object_count = label_order;
  if (image_order >  _object_count )
    _object_count = image_order;
  if (box_order >  _object_count )
    _object_count = box_order;
  if (box_round_order >  _object_count )
    _object_count = box_round_order;
  if (cbox_round_order >  _object_count )
    _object_count = cbox_round_order;
}


static void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_button         = 0;
  exec_label          = 0;
  exec_image          = 0;
  exec_box            = 0;
  exec_round_box      = 0;
  exec_round_cbox     = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnPressPtr != 0) {
          exec_button->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_round_order) {
      if (exec_round_box->Active == 1) {
        if (exec_round_box->OnPressPtr != 0) {
          exec_round_box->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == cbox_round_order) {
      if (exec_round_cbox->Active == 1) {
        if (exec_round_cbox->OnPressPtr != 0) {
          exec_round_cbox->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Button
    case 0: {
      if (PressedObject != 0) {
        exec_button = (TButton*)PressedObject;
        if ((exec_button->PressColEnabled == 1) && (exec_button->OwnerScreen == CurrentScreen)) {
          DrawButton(exec_button);
        }
        break;
      }
      break;
    }
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
        if ((exec_box->PressColEnabled == 1) && (exec_box->OwnerScreen == CurrentScreen)) {
          DrawBox(exec_box);
        }
        break;
      }
      break;
    }
    // Round Box
    case 7: {
      if (PressedObject != 0) {
        exec_round_box = (TBox_Round*)PressedObject;
        if ((exec_round_box->PressColEnabled == 1) && (exec_round_box->OwnerScreen == CurrentScreen)) {
          DrawRoundBox(exec_round_box);
        }
        break;
      }
      break;
    }
    // CRound Box
    case 15: {
      if (PressedObject != 0) {
        exec_round_cbox = (TCBox_Round*)PressedObject;
        if ((exec_round_cbox->PressColEnabled == 1) && (exec_round_cbox->OwnerScreen == CurrentScreen)) {
          DrawCRoundBox(exec_round_cbox);
        }
        break;
      }
      break;
    }
  }

  exec_label          = 0;
  exec_image          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Buttons
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnUpPtr != 0)
          exec_button->OnUpPtr();
        if (PressedObject == (TPointer)exec_button)
          if (exec_button->OnClickPtr != 0)
            exec_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Labels
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (TPointer)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Images
    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (TPointer)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (TPointer)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes with Round Edges
    if (_object_count == box_round_order) {
      if (exec_round_box->Active == 1) {
        if (exec_round_box->OnUpPtr != 0)
          exec_round_box->OnUpPtr();
        if (PressedObject == (TPointer)exec_round_box)
          if (exec_round_box->OnClickPtr != 0)
            exec_round_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // CBoxes with Round Edges
    if (_object_count == cbox_round_order) {
      if (exec_round_cbox->Active == 1) {
        if (exec_round_cbox->OnUpPtr != 0)
          exec_round_cbox->OnUpPtr();
        if (PressedObject == (TPointer)exec_round_cbox)
          if (exec_round_cbox->OnClickPtr != 0)
            exec_round_cbox->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_button         = 0;
  exec_label          = 0;
  exec_image          = 0;
  exec_box            = 0;
  exec_round_box      = 0;
  exec_round_cbox     = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->PressColEnabled == 1) {
          object_pressed = 1;
          DrawButton(exec_button);
        }
        PressedObject = (TPointer)exec_button;
        PressedObjectType = 0;
        if (exec_button->OnDownPtr != 0) {
          exec_button->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (TPointer)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        PressedObject = (TPointer)exec_image;
        PressedObjectType = 3;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->PressColEnabled == 1) {
          object_pressed = 1;
          DrawBox(exec_box);
        }
        PressedObject = (TPointer)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_round_order) {
      if (exec_round_box->Active == 1) {
        if (exec_round_box->PressColEnabled == 1) {
          object_pressed = 1;
          DrawRoundBox(exec_round_box);
        }
        PressedObject = (TPointer)exec_round_box;
        PressedObjectType = 7;
        if (exec_round_box->OnDownPtr != 0) {
          exec_round_box->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == cbox_round_order) {
      if (exec_round_cbox->Active == 1) {
        if (exec_round_cbox->PressColEnabled == 1) {
          object_pressed = 1;
          DrawCRoundBox(exec_round_cbox);
        }
        PressedObject = (TPointer)exec_round_cbox;
        PressedObjectType = 15;
        if (exec_round_cbox->OnDownPtr != 0) {
          exec_round_cbox->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  OSC_CTRL = 0x02;          // 32MHz internal RC oscillator
  while(RC32MRDY_bit == 0)
  ;
  CPU_CCP = 0xD8;
  CLK_CTRL = 1;             // set system clock to 32MHz internal RC oscillator
  TFT_BLED_Direction = 1;
  TFT_BLED = 1;
  TP_TFT_Rotate_180(0);
}

void Init_Ext_Mem() {
  // Initialize SPI
  SPIC_Init_Advanced(_SPI_MASTER,_SPI_FCY_DIV128, _SPI_CLK_LO_LEADING);
  Delay_ms(10);

  // Initialize MMC
  if (!Mmc_Fat_Init()) {
    // Reinitialize SPI at higher speed
    SPIC_Init_Advanced(_SPI_MASTER,_SPI_FCY_DIV4, _SPI_CLK_LO_LEADING);

    // Open resource file for read
    Mmc_Fat_Assign("zu_resou.RES", 0);
    Mmc_Fat_Reset(&res_file_size);
  }
}

void Start_TP() {
  Init_MCU();

  Init_Ext_Mem();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(159, 1830, 139, 1821);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}