#include "zu_objects.h"
#include "zu_resources.h"

//--------------------- User code ---------------------//
// Start time --------------
// Software I2C connections
sbit Soft_I2C_Scl_Output    at PORTD_OUT.B1;
sbit Soft_I2C_Sda_Output    at PORTD_OUT.B0;
sbit Soft_I2C_Scl_Input     at PORTD_IN.B1;
sbit Soft_I2C_Sda_Input     at PORTD_IN.B0;
sbit Soft_I2C_Scl_Direction at PORTD_DIR.B1;
sbit Soft_I2C_Sda_Direction at PORTD_DIR.B0;
// End Software I2C connections
char sek, seconds, minutes, hours, day, weekday, month, year;    // Global date/time variables
char second[16], minute[16], hour[16], days[16], weekdays[16], chislos[16], mecs[16], chiss[16], dogs[16], hhs[16], mms[16], sss[16];
unsigned int chislo, mec, chisl, dog, nedd, vv, hh, mm, ss, abv, c_ear;


void Read_Time() {
  sek = seconds;
  Soft_I2C_Start();               // Issue start signal
  Soft_I2C_Write(0xA0);           // Address PCF8583, see PCF8583 datasheet
  Soft_I2C_Write(2);              // Start from address 2
  Soft_I2C_Start();               // Issue repeated start signal
  Soft_I2C_Write(0xA1);           // Address PCF8583 for reading R/W=1

  seconds = Soft_I2C_Read(1);     // Read seconds byte
  minutes = Soft_I2C_Read(1);     // Read minutes byte
  hours = Soft_I2C_Read(1);       // Read hours byte
  day = Soft_I2C_Read(1);         // Read year/day byte
  month = Soft_I2C_Read(0);       // Read weekday/month byte
  Soft_I2C_Stop();                // Issue stop signal

//-------------------- Formats date and time
  seconds  =  ((seconds & 0xF0) >> 4)*10 + (seconds & 0x0F);  // Transform seconds
  minutes  =  ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);  // Transform months
  hours    =  ((hours & 0xF0)  >> 4)*10  + (hours & 0x0F);    // Transform hours
  year     =   (day & 0xC0) >> 6;                             // Transform year
  day      =  ((day & 0x30) >> 4)*10 + (day & 0x0F);          // Transform day
  weekday  =  ((month & 0xE0) >> 5 );                         // Transform weekday  
  month    =  ((month & 0x10)  >> 4)*10 + (month & 0x0F);     // Transform month 

  if (seconds > sek) { print_time();}
}

// Cyrillic font for RTC
char mese[12][35] = { "\xF1\xCE\xD7\xC1\xD2\xD1","\xE6\xC5\xD7\xD2\xC1\xCC\xD1","\xED\xC1\xD2\xD4\xC1","\xE1\xD0\xD2\xC5\xCC\xD1","\xED\xC1\xD1","\xE9\xC0\xCE\xD1","\xE9\xC0\xCC\xD1","\xE1\xD7\xC7\xD5\xD3\xC1","\xF3\xC5\xCE\xD4\xD1\xC2\xD2\xD1","\xEF\xCB\xD4\xD1\xC2\xD2\xD1","\xEE\xCF\xD1\xC2\xD2\xD1","\xE4\xC5\xCB\xC1\xC2\xD2\xD1" }; // months
char ned[7][9] = { "\xF0\xCE","\xF7\xD4","\xF3\xD2","\xFE\xD4","\xF0\xD4","\xF3\xC2","\xF7\xD3" }; //weekdays
unsigned int yyyy=40, xxxx=21;
void print_time() {
  TFT_Set_Pen(CL_BLACK, 15);
  TFT_Line(xxxx, yyyy+7, xxxx+240, yyyy+7);
  TFT_Set_Ext_Font(Lucida_Console_KOI89x14_Bold, CL_WHITE, FO_HORIZONTAL);
  IntToStr(seconds, second);
  IntToStr(minutes, minute);
  IntToStr(hours, hour);
  IntToStr(day, days);
  TFT_Write_Text(hour, xxxx+135, yyyy);
  TFT_Write_Text(":", xxxx+180, yyyy);
  TFT_Write_Text(minute, xxxx+165, yyyy);
  TFT_Write_Text(":", xxxx+210, yyyy);
  TFT_Write_Text(second, xxxx+195, yyyy);
  TFT_Write_Text(days, xxxx-20, yyyy);
  TFT_Write_Text(&mese[month-1], xxxx+35, yyyy);
  TFT_Write_Text(&ned[weekday], xxxx+105, yyyy);
}

void Write_Time(unsigned char c_hours, unsigned char c_minutes, unsigned char c_seconds, unsigned char c_date, unsigned char c_month, unsigned char c_year, unsigned char c_wday) {

// c_year = c_year;
c_year = (c_year % 4) << 6;
c_wday = (c_wday) << 5;
   Soft_I2C_Start();      // Issue start signal
   Soft_I2C_Write(0xA0);  // Address PCF8583 datasheet
   Soft_I2C_Write(0);     // Start from address 0 (configuration memory location)
   Soft_I2C_Write(0x80);  // Write 0x80 to configuration memory location (pause counter...)
   Soft_I2C_Write(0);     // Write 0 to cents memory location
   Soft_I2C_Write(c_seconds);     // Write 0 to seconds memory location
   Soft_I2C_Write(c_minutes);  // Write 0x30 to minutes memory location
   Soft_I2C_Write(c_hours);  // Write 0x9 to hours memory location
   Soft_I2C_Write(c_date | c_year);  // Write 0x24 to date/year memory location
   Soft_I2C_Write(c_month | c_wday);  // Write 0x08 to weekday/month memory location
   Soft_I2C_Stop();       // Issue stop signal

   Soft_I2C_Start();      // Issue start signal
   Soft_I2C_Write(0xA0);  // Address PCF8583
   Soft_I2C_Write(0);     // Start from address 0
   Soft_I2C_Write(0);     // Write 0 to configuration memory location (enable counting)
   Soft_I2C_Stop();       // Issue stop signal

}

//   end time -----------------------------------

//----------------- End of User code ------------------//


// Event Handlers
unsigned short yyy = 66;
unsigned short xxx = 20;
unsigned short xxz = 12;
unsigned short zzz = 0;
unsigned short rrr;
unsigned short chis;
unsigned short mes;
unsigned int god = 2000;
unsigned short ch;
unsigned short mi;
unsigned short se;
char ttt[4];
void nomer();

void setting() {
     DrawScreen(&Screen2);

}

void clock() {
     DrawScreen(&Screen3);
     TFT_Set_Ext_Font(Lucida_Console_KOI89x14_Bold, CL_WHITE, FO_HORIZONTAL);
     TFT_Write_Text("\xF5\xD3\xD4\xC1\xCE\xCF\xD7\xCB\xC1 \xF7\xD2\xC5\xCD\xC5\xCE\xC9", 85, 20);
     // TFT_Write_Text("31-12-14 23:59:59", 90, 40);
}

void lamp() {
     DrawScreen(&Screen4);
     TFT_Set_Ext_Font(Lucida_Console_KOI89x14_Bold, CL_WHITE, FO_HORIZONTAL);
     TFT_Write_Text("\xF5\xD3\xD4\xC1\xCE\xCF\xD7\xCB\xC1 \xF1\xD2\xCB\xCF\xD3\xD4\xC9", 85, 29);
}

void exit() {
     DrawScreen(&Screen1);
}

void exit2() {
     DrawScreen(&Screen2);
}

void exit3() {
     DrawScreen(&Screen2);
}

void minus() {

}

void plus() {

}

void esc() {
     zzz = 0;
     xxx = 20;
     TFT_Set_Pen(CL_SILVER, 28);
     TFT_H_Line(12, 307, 73);
}

void enter() {
     Read_Time();
}

void nomer() {
     if (zzz != 0) {xxx = xxx + xxz;}
     ByteToStr(rrr, ttt);
     TFT_Set_Ext_Font(Lucida_Console_KOI812x20_Bold, CL_BLACK, FO_HORIZONTAL);
     if ((zzz == 2) || (zzz == 4)) {xxx+= xxz + 2; TFT_Write_Char('-', xxx, yyy); xxx-= 2; }
     if (zzz == 4) {xxx+= xxz; TFT_Write_Char('2', xxx, yyy); xxx+= xxz; TFT_Write_Char('0', xxx, yyy); xxx-= 2; }
     if (zzz == 6) {xxx+= xxz + 20; }
     if ((zzz == 8) || (zzz == 10)) {xxx+= xxz + 2; TFT_Write_Char(':', xxx, yyy); xxx-= 2; }
     TFT_Write_Text(ttt, xxx, yyy);
     zzz++;
     if (zzz > 12) {esc();}
}

void odin() {
     rrr=1;
     if (zzz == 0) {chis = 10 ;}
     if (zzz == 1) {chis+=1 ;}
     if (zzz == 2) {mes = 10 ;}
     if (zzz == 3) {mes+= 1 ;}
     if (zzz == 4) {god = 2010 ;}
     if (zzz == 5) {god+= 1 ;}
     if (zzz == 6) {ch = 10 ;}
     if (zzz == 7) {ch+= 1 ;}
     if (zzz == 8) {mi = 10 ;}
     if (zzz == 9) {mi+= 1 ;}
     if (zzz == 10) {se = 10 ;}
     if (zzz == 11) {se+= 1 ;}
     nomer();
}

void dva() {
     rrr=2;
     if (zzz == 0) {chis = 20; }
     if (zzz == 1) {chis+= 2; rrr=2;}
     if (zzz == 2) {mes = 10; rrr=1;}
     if (zzz == 3) {mes+= 2;}
     if (zzz == 4) {god = 2020;}
     if (zzz == 5) {god+= 2;}
     if (zzz == 6) {ch = 20;}
     if (zzz == 7) {ch+= 2;}
     if (zzz == 8) {mi = 20;}
     if (zzz == 9) {mi+= 2 ;}
     if (zzz == 10) {se = 20 ;}
     if (zzz == 11) {se+= 2 ;}
     nomer();
}

void tri() {
     rrr=3;
     if (zzz == 0) {chis = 30; }
     if (zzz == 1) {chis+= 3;}
     // if (chis > 31) {chis = 31; rrr=1;}
     if (zzz == 2) {mes = 10; rrr=1;}
     if (zzz == 3) {mes += 3;}
     // if (mes > 12) {mes = 12; rrr=2;}
     if (zzz == 4) {god = 2030;}
     if (zzz == 5) {god += 3;}
     if (zzz == 6) {ch = 20; rrr=2;}
     if (zzz == 7) {ch += 3;}
     // if (ch > 23) {ch = 23; rrr=3;}
     if (zzz == 8) {mi = 30;}
     if (zzz == 9) {mi += 3 ;}
     // if (mi > 59) {mi = 59; rrr=9;}
     if (zzz == 10) {se = 30 ;}
     if (zzz == 11) {se += 3 ;}
     // if (se > 59) {se = 59; rrr=9;}
     nomer();
}

void chetire() {
     rrr=4;
     if (zzz == 0) {chis = 30; rrr=3; }
     if (zzz == 1) {chis += 4;}
     // if (chis > 31) {chis = 31; rrr=1;}
     if (zzz == 2) {mes = 10; rrr=1;}
     if (zzz == 3) {mes += 4;}
     // if (mes > 12) {mes = 12; rrr=2;}
     if (zzz == 4) {god = 2040;}
     if (zzz == 5) {god += 4;}
     if (zzz == 6) {ch = 20; rrr=2;}
     if (zzz == 7) {ch += 4;}
     // if (ch > 23) {ch = 23; rrr=3;}
     if (zzz == 8) {mi = 40;}
     if (zzz == 9) {mi += 4 ;}
     // if (mi > 59) {mi = 59; rrr=9;}
     if (zzz == 10) {se = 40 ;}
     if (zzz == 11) {se += 4 ;}
     // if (se > 59) {se = 59; rrr=9;}
     nomer();
}

void pyat() {
     rrr=5;
     if (zzz == 0) {chis = 30; rrr=3; }
     if (zzz == 1) {chis += 5;}
     // if (chis > 31) {chis = 31; rrr=1;}
     if (zzz == 2) {mes = 10; rrr=1;}
     if (zzz == 3) {mes += 5;}
     // if (mes > 12) {mes = 12; rrr=2;}
     if (zzz == 4) {god = 2050;}
     if (zzz == 5) {god += 5;}
     if (zzz == 6) {ch = 20; rrr=2;}
     if (zzz == 7) {ch += 5;}
     // if (ch > 23) {ch = 23; rrr=3;}
     if (zzz == 8) {mi = 50;}
     if (zzz == 9) {mi += 5 ;}
     // if (mi > 59) {mi = 59; rrr=9;}
     if (zzz == 10) {se = 50 ;}
     if (zzz == 11) {se += 5 ;}
     // if (se > 59) {se = 59; rrr=9;}
     nomer();
}

void chest() {
     rrr=6;
     if (zzz == 0) {chis = 30; rrr=3; }
     if (zzz == 1) {chis += 6;}
     // if (chis > 31) {chis = 31; rrr=1;}
     if (zzz == 2) {mes = 10; rrr=1;}
     if (zzz == 3) {mes += 6;}
     // if (mes > 12) {mes = 12; rrr=2;}
     if (zzz == 4) {god = 2060;}
     if (zzz == 5) {god += 6;}
     if (zzz == 6) {ch = 20; rrr=2;}
     if (zzz == 7) {ch += 6;}
     // if (ch > 23) {ch = 23; rrr=3;}
     if (zzz == 8) {mi = 50; rrr=5;}
     if (zzz == 9) {mi += 6 ;}
     // if (mi > 59) {mi = 59; rrr=9;}
     if (zzz == 10) {se = 50; rrr=5;}
     if (zzz == 11) {se += 6 ;}
     // if (se > 59) {se = 59; rrr=9;}
     nomer();
}

void sem() {
     rrr=7;
     if (zzz == 0) {chis = 30; rrr=3; }
     if (zzz == 1) {chis += 7;}
     // if (chis > 31) {chis = 31; rrr=1;}
     if (zzz == 2) {mes = 10; rrr=1;}
     if (zzz == 3) {mes += 7;}
     // if (mes > 12) {mes = 12; rrr=2;}
     if (zzz == 4) {god = 2070;}
     if (zzz == 5) {god += 7;}
     if (zzz == 6) {ch = 20; rrr=2;}
     if (zzz == 7) {ch += 7;}
     // if (ch > 23) {ch = 23; rrr=3;}
     if (zzz == 8) {mi = 50; rrr=5;}
     if (zzz == 9) {mi += 7 ;}
     // if (mi > 59) {mi = 59; rrr=9;}
     if (zzz == 10) {se = 50; rrr=5;}
     if (zzz == 11) {se += 7 ;}
     // if (se > 59) {se = 59; rrr=9;}
     nomer();
}

void vosem() {
     rrr=8;
     if (zzz == 0) {chis = 30; rrr=3; }
     if (zzz == 1) {chis += 8;}
     // if (chis > 31) {chis = 31; rrr=1;}
     if (zzz == 2) {mes = 10; rrr=1;}
     if (zzz == 3) {mes += 8;}
     // if (mes > 12) {mes = 12; rrr=2;}
     if (zzz == 4) {god = 2080;}
     if (zzz == 5) {god += 8;}
     if (zzz == 6) {ch = 20; rrr=2;}
     if (zzz == 7) {ch += 8;}
     // if (ch > 23) {ch = 23; rrr=3;}
     if (zzz == 8) {mi = 50; rrr=5;}
     if (zzz == 9) {mi += 8 ;}
     // if (mi > 59) {mi = 59; rrr=9;}
     if (zzz == 10) {se = 50; rrr=5;}
     if (zzz == 11) {se += 8 ;}
     // if (se > 59) {se = 59; rrr=9;}
     nomer();
}

void devyat() {
     rrr=9;
     if (zzz == 0) {chis = 30; rrr=3; }
     if (zzz == 1) {chis += 9;}
     // if (chis > 31) {chis = 31; rrr=1;}
     if (zzz == 2) {mes = 10; rrr=1;}
     if (zzz == 3) {mes += 9;}
     // if (mes > 12) {mes = 12; rrr=2;}
     if (zzz == 4) {god = 2090;}
     if (zzz == 5) {god += 9;}
     if (zzz == 6) {ch = 20; rrr=2;}
     if (zzz == 7) {ch += 9;}
     // if (ch > 23) {ch = 23; rrr=3;}
     if (zzz == 8) {mi = 50; rrr=5;}
     if (zzz == 9) {mi += 9 ;}
     // if (mi > 59) {mi = 59; rrr=9;}
     if (zzz == 10) {se = 50; rrr=5;}
     if (zzz == 11) {se += 9 ;}
     // if (se > 59) {se = 59; rrr=9;}
     nomer();
}

void nol() {
     rrr=0;
     // if (zzz == 4) {god = 2000;}
     nomer();
}